HOVER_deepSchedule = function(f, vars, t, rep, st)
local functionID

	local deepVars = mist.utils.deepCopy(vars)
	functionID = mist.scheduleFunction(f, deepVars, t, rep, st)
	return functionID
end

HOVER_out = function(coa,outText,outTime) -- function outputs text using the mist message system

	if outText and outTime then
		local msg = {}
		msg.text = outText
		msg.displayTime = outTime
		if coa == coalition.side.RED then
			msg.msgFor = {coa = {'red'}}
		else
			msg.msgFor = {coa = {'blue'}}
		end
		msg.sound = 'RadioClickAfter.wav'
		mist.message.add(msg)
	end
end

HOVER_addGroundGroup = function(playerUnit)
local playerPosition,playerHeading
local units = {}

	if playerUnit then
		HOVER_groupCounter = HOVER_groupCounter+1
		playerPosition = Unit.getPosition(playerUnit).p
		playerHeading = math.atan2(Unit.getPosition(playerUnit).x.z,Unit.getPosition(playerUnit).x.x)
		table.insert(units,
		{
			["x"] = playerPosition.x-5*math.sin(playerHeading),
			["y"] = playerPosition.z+5*math.cos(playerHeading),
			["type"] = 'hello',
			["name"] = 'Special Forces Group #' .. HOVER_groupCounter .. ' Unit #' .. 1,
			["heading"] = 0,
			["skill"] = "Random"
		})
		if Unit.getCoalition(playerUnit) == coalition.side.RED then
			units[1]["type"] = 'Infantry AK'
			mist.dynAdd({country = 'RUSSIA', category = 'GROUND_UNIT', name = 'Special Forces Group #' .. HOVER_groupCounter, hidden = false, units = units })
		else
			units[1]["type"] = 'Soldier M4'
			mist.dynAdd({country = 'USA', category = 'GROUND_UNIT', name = 'Special Forces Group #' .. HOVER_groupCounter, hidden = false, units = units })
		end
		HOVER_out(Unit.getCoalition(playerUnit),'Boots are on the ground.',2)
	end
end

HOVER_guidance = function(playerName)
local count
local playerPosition
local playerUnit
local playerHeading
local relativePosition
local headingVector
local headingPerpendicular
local forwardDistance
local rightDistance
local landHeight
local distance,velocity,speed
local hoverPosition,minDistance,minCount
local groupId

	playerUnit = Unit.getByName(playerName)
	if playerUnit then
		groupId = Group.getID(Unit.getGroup(playerUnit))
		playerPosition = Unit.getPosition(playerUnit).p
		landHeight = land.getHeight({x=playerPosition.x,y=playerPosition.z})
		minDistance = nil
		for count = 1,#HOVER_points do
			if not minDistance or mist.utils.get2DDist(playerPosition,HOVER_points[count].position) < minDistance then
				minCount = count
				minDistance = mist.utils.get2DDist(playerPosition,HOVER_points[count].position)
			end
		end
		if minDistance then
			hoverPosition = HOVER_points[minCount].position
			playerHeading = math.atan2(Unit.getPosition(playerUnit).x.z,Unit.getPosition(playerUnit).x.x)
			headingVector = {x = math.cos(playerHeading),y=0,z=math.sin(playerHeading)}
			headingVectorPerpendicular = {x = -math.sin(playerHeading),y=0,z=math.cos(playerHeading)}
			relativePosition = mist.vec.sub(hoverPosition,playerPosition)
			forwardDistance = mist.vec.dp(relativePosition,headingVector)
			rightDistance = mist.vec.dp(relativePosition,headingVectorPerpendicular)
			distance = math.sqrt(forwardDistance*forwardDistance+rightDistance*rightDistance)
			-- OIL_out('player heading ' .. playerHeading .. '\nrelative vector ' .. relativePosition.x .. ', '  .. relativePosition.y .. ', ' .. relativePosition.z .. '\nheading vector ' .. headingVector.x .. ', '  .. headingVector.y .. ', ' .. headingVector.z .. '\nforward distance ' .. forwardDistance .. '\nright distance ' .. rightDistance,2)
			if distance < HOVER_points[minCount].minD then
				HOVER_points[minCount].duration = HOVER_points[minCount].duration+2
			elseif distance > HOVER_points[minCount].minD and distance < HOVER_points[minCount].maxD then
				HOVER_points[minCount].duration = 0
				if math.abs(forwardDistance) > math.abs(rightDistance) then
					trigger.action.outTextForGroup(groupId,math.floor(math.abs(forwardDistance)) .. 'm',2)
					if forwardDistance > 0 then
						trigger.action.outSoundForGroup(groupId,'forward.wav')
					else
						trigger.action.outSoundForGroup(groupId,'back.wav')
					end
				else
					trigger.action.outTextForGroup(groupId,math.floor(math.abs(rightDistance)) .. 'm',2)
					if rightDistance > 0 then
						trigger.action.outSoundForGroup(groupId,'right.wav')
					else
						trigger.action.outSoundForGroup(groupId,'left.wav')
					end
				end
			end
			if distance < 10 then
				if playerPosition.y > landHeight + HOVER_points[minCount].maxH then
					HOVER_points[minCount].duration = 0
					HOVER_deepSchedule(trigger.action.outSoundForGroup,{groupId,'down.wav'},timer.getTime()+1)
				elseif playerPosition.y < landHeight + HOVER_points[minCount].minH then
					HOVER_points[minCount].duration = 0
					HOVER_deepSchedule(trigger.action.outSoundForGroup,{groupId,'up.wav'},timer.getTime()+1)
				end
			end
			if HOVER_points[minCount].duration > 1 then
				if HOVER_troopsInCockpit['Player'] > 0 then
					HOVER_troopsInCockpit['Player'] = HOVER_troopsInCockpit['Player']-1
					HOVER_out(Unit.getCoalition(playerUnit),'Go, go, go',2)
					HOVER_deepSchedule(HOVER_addGroundGroup,{playerUnit},timer.getTime()+3)
				end
			end
		end
	end
end

HOVER_smokeMarker = function()
local count
local smokePosition

	for count = 1,#HOVER_points do
		landHeight = land.getHeight({x=HOVER_points[count].position.x,y=HOVER_points[count].position.z})
		smokePosition = {x=HOVER_points[count].position.x,y=landHeight+5,z=HOVER_points[count].position.z}
		trigger.action.smoke(smokePosition, 0)
	end
end

HOVER_points = {}
HOVER_groupCounter = 0
HOVER_troopsInCockpit = {}

-- BEGIN MAIN
table.insert(HOVER_points,{position = mist.utils.zoneToVec3('Hover Zone'),minD = 10,maxD = 300,minH = 20,maxH = 40, duration = 0})
HOVER_troopsInCockpit['Player'] = 8
HOVER_deepSchedule(HOVER_guidance,{'Player'},timer.getTime()+1,2)
HOVER_deepSchedule(HOVER_smokeMarker,{},timer.getTime()+1,180)
