HOVER_deepSchedule = function(f, vars, t, rep, st)
local functionID

	local deepVars = mist.utils.deepCopy(vars)
	functionID = mist.scheduleFunction(f, deepVars, t, rep, st)
	return functionID
end

HOVER_out = function(coa,outText,outTime) -- function outputs text using the mist message system

	if outText and outTime then
		local msg = {}
		msg.text = outText
		msg.displayTime = outTime
		if coa == coalition.side.RED then
			msg.msgFor = {coa = {'red'}}
		else
			msg.msgFor = {coa = {'blue'}}
		end
		msg.sound = 'RadioClickAfter.wav'
		mist.message.add(msg)
	end
end

addGroundGroup = function (country,groupName,unitType,numberOfTroops,position)
local i
local units = {}
local unitPosition

	for i =1,numberOfTroops do
		unitPosition = mist.getRandPointInCircle(mist.utils.makeVec3(position), 1.5*math.sqrt(numberOfTroops)) -- get random position for individual unit
		table.insert(units,
		{
			["x"] = unitPosition.x,
			["y"] = unitPosition.y,
			["type"] = unitType,
			["name"] = groupName .. ' Unit #' .. i,
			["heading"] = 0,
			["skill"] = "Random"
		})
	end
	if #units > 0 then
		mist.dynAdd({country = country, category = 'GROUND_UNIT', name = groupName, hidden = false, units = units })
		return groupName
	else
		return nil
	end
end

HOVER_fastRope = function(playerUnit)
local playerPosition
local units = {}
local minDistance,count,minCount

	if playerUnit then
		playerPosition = Unit.getPosition(playerUnit).p
		minDistance = nil
		for count = 1,#HOVER_points do
			if not minDistance or mist.utils.get2DDist(playerPosition,HOVER_points[count].position) < minDistance then
				minCount = count
				minDistance = mist.utils.get2DDist(playerPosition,HOVER_points[count].position)
			end
		end
		if minDistance then
			if HOVER_points[minCount].duration > 1 then
				HOVER_groupCounter = HOVER_groupCounter+1
				nextOne = true
				HOVER_out(Unit.getCoalition(playerUnit),'Unit ' .. HOVER_groupCounter .. ' is on the ground.',2)
				if HOVER_troopsInCabin[Unit.getName(playerUnit)] == 0 then
					if Unit.getCoalition(playerUnit) == coalition.side.RED then
						addGroundGroup('RUSSIA','Special Forces Group','Infantry AK',HOVER_fullCabin[Unit.getName(playerUnit)],HOVER_points[minCount].troopZone)
					else
						addGroundGroup('USA','Special Forces Group','Soldier M4',HOVER_fullCabin[Unit.getName(playerUnit)],HOVER_points[minCount].troopZone)
					end
					trigger.action.setUserFlag('99',true)
				end
			end
		end
		if not nextOne then
			HOVER_deepSchedule(HOVER_fastRope,{playerUnit},timer.getTime()+1)
		end
	end
end

HOVER_guidance = function(playerName)
local count
local playerPosition
local playerUnit
local playerHeading
local relativePosition
local headingVector
local headingPerpendicular
local forwardDistance
local rightDistance
local landHeight
local distance,velocity,speed
local hoverPosition,minDistance,minCount
local groupId

	playerUnit = Unit.getByName(playerName)
	if playerUnit then
		groupId = Group.getID(Unit.getGroup(playerUnit))
		playerPosition = Unit.getPosition(playerUnit).p
		landHeight = land.getHeight({x=playerPosition.x,y=playerPosition.z})
		minDistance = nil
		for count = 1,#HOVER_points do
			if not minDistance or mist.utils.get2DDist(playerPosition,HOVER_points[count].position) < minDistance then
				minCount = count
				minDistance = mist.utils.get2DDist(playerPosition,HOVER_points[count].position)
			end
		end
		if minDistance then
			hoverPosition = HOVER_points[minCount].position
			playerHeading = math.atan2(Unit.getPosition(playerUnit).x.z,Unit.getPosition(playerUnit).x.x)
			headingVector = {x = math.cos(playerHeading),y=0,z=math.sin(playerHeading)}
			headingVectorPerpendicular = {x = -math.sin(playerHeading),y=0,z=math.cos(playerHeading)}
			relativePosition = mist.vec.sub(hoverPosition,playerPosition)
			forwardDistance = mist.vec.dp(relativePosition,headingVector)
			rightDistance = mist.vec.dp(relativePosition,headingVectorPerpendicular)
			distance = math.sqrt(forwardDistance*forwardDistance+rightDistance*rightDistance)
			-- OIL_out('player heading ' .. playerHeading .. '\nrelative vector ' .. relativePosition.x .. ', '  .. relativePosition.y .. ', ' .. relativePosition.z .. '\nheading vector ' .. headingVector.x .. ', '  .. headingVector.y .. ', ' .. headingVector.z .. '\nforward distance ' .. forwardDistance .. '\nright distance ' .. rightDistance,2)
			if distance < HOVER_points[minCount].minD then
				HOVER_points[minCount].duration = HOVER_points[minCount].duration+2
			elseif distance > HOVER_points[minCount].minD and distance < HOVER_points[minCount].maxD then
				HOVER_points[minCount].duration = 0
				if math.abs(forwardDistance) > math.abs(rightDistance) then
					trigger.action.outTextForGroup(groupId,math.floor(math.abs(forwardDistance)) .. 'm',2)
					if forwardDistance > 0 then
						trigger.action.outSoundForGroup(groupId,'forward.wav')
					else
						trigger.action.outSoundForGroup(groupId,'back.wav')
					end
				else
					trigger.action.outTextForGroup(groupId,math.floor(math.abs(rightDistance)) .. 'm',2)
					if rightDistance > 0 then
						trigger.action.outSoundForGroup(groupId,'right.wav')
					else
						trigger.action.outSoundForGroup(groupId,'left.wav')
					end
				end
			end
			if distance < HOVER_points[minCount].maxD then
				if playerPosition.y > landHeight + HOVER_points[minCount].maxH then
					HOVER_points[minCount].duration = 0
					HOVER_deepSchedule(trigger.action.outSoundForGroup,{groupId,'down.wav'},timer.getTime()+1)
				elseif playerPosition.y < landHeight + HOVER_points[minCount].minH then
					HOVER_points[minCount].duration = 0
					HOVER_deepSchedule(trigger.action.outSoundForGroup,{groupId,'up.wav'},timer.getTime()+1)
				end
			end
			if HOVER_points[minCount].duration > 1 then
				if nextOne and HOVER_troopsInCabin['Player'] > 0 then
					HOVER_troopsInCabin['Player'] = HOVER_troopsInCabin['Player']-1
					HOVER_out(Unit.getCoalition(playerUnit),'Go, go, go',2)
					nextOne = false
					HOVER_deepSchedule(HOVER_fastRope,{playerUnit},timer.getTime()+3)
				end
			end
		end
	end
	if HOVER_troopsInCabin[playerName] > 0 then
		HOVER_deepSchedule(HOVER_guidance,{playerName},timer.getTime()+2)
	end
end

-- BEGIN MAIN

HOVER_points = {}
HOVER_groupCounter = 0
HOVER_troopsInCabin = {}
HOVER_fullCabin = {}
nextOne = true

table.insert(HOVER_points,{position = mist.utils.zoneToVec3('Hover Zone'),troopZone = mist.utils.zoneToVec3('Troup Zone'),minD = 10,maxD = 300,minH = 70,maxH = 90, duration = 0})
HOVER_troopsInCabin['Player'] = 10
HOVER_fullCabin['Player'] = 10
trigger.action.setUserFlag('99',false)
HOVER_deepSchedule(HOVER_guidance,{'Player'},timer.getTime()+2)
